@@ -1,15 +1,17 @@
 from __future__ import print_function
+from __future__ import absolute_import
 import argparse
 import logging
 import sys
 import traceback
 import json
 import os
-import urlparse
+
+from six.moves import urllib
 
 import pkg_resources  # part of setuptools
 
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Union, Text
 
 from rdflib import Graph, plugin
 from rdflib.serializer import Serializer
@@ -20,6 +22,7 @@ from . import makedoc
 from . import validate
 from .sourceline import strip_dup_lineno
 from .ref_resolver import Loader
+import six
 
 _logger = logging.getLogger("salad")
 
@@ -28,8 +31,8 @@ register('json-ld', Parser, 'rdflib_jsonld.parser', 'JsonLDParser')
 
 
 def printrdf(workflow,  # type: str
-             wf,        # type: Union[List[Dict[unicode, Any]], Dict[unicode, Any]]
-             ctx,       # type: Dict[unicode, Any]
+             wf,        # type: Union[List[Dict[Text, Any]], Dict[Text, Any]]
+             ctx,       # type: Dict[Text, Any]
              sr         # type: str
              ):
     # type: (...) -> None
@@ -62,8 +65,6 @@ def main(argsl=None):  # type: (List[str]) -> int
         "--print-index", action="store_true", help="Print node index")
     exgroup.add_argument("--print-metadata",
                          action="store_true", help="Print document metadata")
-    exgroup.add_argument("--version", action="store_true",
-                         help="Print version")
 
     exgroup = parser.add_mutually_exclusive_group()
     exgroup.add_argument("--strict", action="store_true", help="Strict validation (unrecognized or out of place fields are error)",
@@ -79,11 +80,18 @@ def main(argsl=None):  # type: (List[str]) -> int
     exgroup.add_argument("--debug", action="store_true",
                          help="Print even more logging")
 
-    parser.add_argument("schema", type=str)
+    parser.add_argument("schema", type=str, nargs="?", default=None)
     parser.add_argument("document", type=str, nargs="?", default=None)
+    parser.add_argument("--version", "-v", action="store_true",
+                        help="Print version", default=None)
+
 
     args = parser.parse_args(argsl)
 
+    if args.version is None and args.schema is None:
+        print('%s: error: too few arguments' % sys.argv[0])
+        return 1
+
     if args.quiet:
         _logger.setLevel(logging.WARN)
     if args.debug:
@@ -92,10 +100,10 @@ def main(argsl=None):  # type: (List[str]) -> int
     pkg = pkg_resources.require("schema_salad")
     if pkg:
         if args.version:
-            print("%s %s" % (sys.argv[0], pkg[0].version))
+            print("%s Current version: %s" % (sys.argv[0], pkg[0].version))
             return 0
         else:
-            _logger.info("%s %s", sys.argv[0], pkg[0].version)
+            _logger.info("%s Current version: %s", sys.argv[0], pkg[0].version)
 
     # Get the metaschema to validate the schema
     metaschema_names, metaschema_doc, metaschema_loader = schema.get_metaschema()
@@ -103,7 +111,7 @@ def main(argsl=None):  # type: (List[str]) -> int
     # Load schema document and resolve refs
 
     schema_uri = args.schema
-    if not urlparse.urlparse(schema_uri)[0]:
+    if not urllib.parse.urlparse(schema_uri)[0]:
         schema_uri = "file://" + os.path.abspath(schema_uri)
     schema_raw_doc = metaschema_loader.fetch(schema_uri)
 
@@ -113,8 +121,8 @@ def main(argsl=None):  # type: (List[str]) -> int
     except (validate.ValidationException) as e:
         _logger.error("Schema `%s` failed link checking:\n%s",
                       args.schema, e, exc_info=(True if args.debug else False))
-        _logger.debug("Index is %s", metaschema_loader.idx.keys())
-        _logger.debug("Vocabulary is %s", metaschema_loader.vocab.keys())
+        _logger.debug("Index is %s", list(metaschema_loader.idx.keys()))
+        _logger.debug("Vocabulary is %s", list(metaschema_loader.vocab.keys()))
         return 1
     except (RuntimeError) as e:
         _logger.error("Schema `%s` read error:\n%s",
@@ -127,14 +135,14 @@ def main(argsl=None):  # type: (List[str]) -> int
         return 0
 
     if not args.document and args.print_index:
-        print(json.dumps(metaschema_loader.idx.keys(), indent=4))
+        print(json.dumps(list(metaschema_loader.idx.keys()), indent=4))
         return 0
 
     # Validate the schema document against the metaschema
     try:
         schema.validate_doc(metaschema_names, schema_doc,
                             metaschema_loader, args.strict,
-                            source=schema_metadata["name"])
+                            source=schema_metadata.get("name"))
     except validate.ValidationException as e:
         _logger.error("While validating schema `%s`:\n%s" %
                       (args.schema, str(e)))
@@ -146,7 +154,11 @@ def main(argsl=None):  # type: (List[str]) -> int
         metactx = schema_raw_doc.get("$namespaces", {})
         if "$base" in schema_raw_doc:
             metactx["@base"] = schema_raw_doc["$base"]
-    (schema_ctx, rdfs) = jsonld_context.salad_to_jsonld_context(schema_doc, metactx)
+    if schema_doc is not None:
+        (schema_ctx, rdfs) = jsonld_context.salad_to_jsonld_context(
+            schema_doc, metactx)
+    else:
+        raise Exception("schema_doc is None??")
 
     # Create the loader that will be used to load the target document.
     document_loader = Loader(schema_ctx)
@@ -196,12 +208,12 @@ def main(argsl=None):  # type: (List[str]) -> int
     # Load target document and resolve refs
     try:
         uri = args.document
-        if not urlparse.urlparse(uri)[0]:
+        if not urllib.parse.urlparse(uri)[0]:
             doc = "file://" + os.path.abspath(uri)
         document, doc_metadata = document_loader.resolve_ref(uri)
     except (validate.ValidationException, RuntimeError) as e:
         _logger.error("Document `%s` failed validation:\n%s",
-                      args.document, strip_dup_lineno(unicode(e)), exc_info=args.debug)
+                      args.document, strip_dup_lineno(six.text_type(e)), exc_info=args.debug)
         return 1
 
     # Optionally print the document after ref resolution
@@ -210,7 +222,7 @@ def main(argsl=None):  # type: (List[str]) -> int
         return 0
 
     if args.print_index:
-        print(json.dumps(document_loader.idx.keys(), indent=4))
+        print(json.dumps(list(document_loader.idx.keys()), indent=4))
         return 0
 
     # Validate the schema document against the metaschema
