@@ -1,6 +1,7 @@
+from __future__ import absolute_import
 import avro
 import copy
-from .add_dictlist import add_dictlist
+from schema_salad.utils import add_dictlist, aslist, flatten
 import sys
 import pprint
 from pkg_resources import resource_stream
@@ -8,19 +9,23 @@ import ruamel.yaml as yaml
 import avro.schema
 from . import validate
 import json
-import urlparse
 import os
-AvroSchemaFromJSONData = avro.schema.make_avsc_object
-# AvroSchemaFromJSONData=avro.schema.SchemaFromJSONData
+
+import six
+from six.moves import urllib
+
+if six.PY3:
+    AvroSchemaFromJSONData = avro.schema.SchemaFromJSONData
+else:
+    AvroSchemaFromJSONData = avro.schema.make_avsc_object
+
 from avro.schema import Names, SchemaParseException
 from . import ref_resolver
 from .ref_resolver import Loader, DocumentType
-from .flatten import flatten
 import logging
-from .aslist import aslist
 from . import jsonld_context
 from .sourceline import SourceLine, strip_dup_lineno, add_lc_filename, bullets, relname
-from typing import Any, AnyStr, cast, Dict, List, Tuple, TypeVar, Union
+from typing import cast, Any, AnyStr, Dict, List, Set, Tuple, TypeVar, Union, Text
 from ruamel.yaml.comments import CommentedSeq, CommentedMap
 
 _logger = logging.getLogger("salad")
@@ -45,11 +50,19 @@ salad_files = ('metaschema.yml',
                'link_res_proc.yml',
                'vocab_res_schema.yml',
                'vocab_res_src.yml',
-               'vocab_res_proc.yml')
+               'vocab_res_proc.yml',
+               'map_res.yml',
+               'map_res_schema.yml',
+               'map_res_src.yml',
+               'map_res_proc.yml',
+               'typedsl_res.yml',
+               'typedsl_res_schema.yml',
+               'typedsl_res_src.yml',
+               'typedsl_res_proc.yml')
 
 
 def get_metaschema():
-    # type: () -> Tuple[Names, List[Dict[unicode, Any]], Loader]
+    # type: () -> Tuple[Names, List[Dict[Text, Any]], Loader]
     loader = ref_resolver.Loader({
         "Any": "https://w3id.org/cwl/salad#Any",
         "ArraySchema": "https://w3id.org/cwl/salad#ArraySchema",
@@ -163,7 +176,7 @@ def get_metaschema():
     loader.cache["https://w3id.org/cwl/salad"] = rs.read()
     rs.close()
 
-    j = yaml.round_trip_load(loader.cache["https://w3id.org/cwl/salad"])  # type: ignore
+    j = yaml.round_trip_load(loader.cache["https://w3id.org/cwl/salad"])
     add_lc_filename(j, "metaschema.yml")
     j, _ = loader.resolve_all(j, "https://w3id.org/cwl/salad#")
 
@@ -178,10 +191,10 @@ def get_metaschema():
     return (sch_names, j, loader)
 
 
-def load_schema(schema_ref,  # type: Union[CommentedMap, CommentedSeq, unicode]
+def load_schema(schema_ref,  # type: Union[CommentedMap, CommentedSeq, Text]
                 cache=None   # type: Dict
                 ):
-    # type: (...) -> Tuple[Loader, Union[Names, SchemaParseException], Dict[unicode, Any], Loader]
+    # type: (...) -> Tuple[Loader, Union[Names, SchemaParseException], Dict[Text, Any], Loader]
     """Load a schema that can be used to validate documents using load_and_validate.
 
     return document_loader, avsc_names, schema_metadata, metaschema_loader"""
@@ -212,10 +225,10 @@ def load_schema(schema_ref,  # type: Union[CommentedMap, CommentedSeq, unicode]
 
 def load_and_validate(document_loader,  # type: Loader
                       avsc_names,       # type: Names
-                      document,         # type: Union[CommentedMap, unicode]
+                      document,         # type: Union[CommentedMap, Text]
                       strict            # type: bool
                       ):
-    # type: (...) -> Tuple[Any, Dict[unicode, Any]]
+    # type: (...) -> Tuple[Any, Dict[Text, Any]]
     """Load a document and validate it with the provided schema.
 
     return data, metadata
@@ -234,23 +247,23 @@ def load_and_validate(document_loader,  # type: Loader
 
     validationErrors = u""
     try:
-        document_loader.validate_links(data, u"")
+        document_loader.validate_links(data, u"", {})
     except validate.ValidationException as v:
-        validationErrors = unicode(v) + "\n"
+        validationErrors = six.text_type(v) + "\n"
 
     try:
         validate_doc(avsc_names, data, document_loader, strict, source=source)
     except validate.ValidationException as v:
-        validationErrors += unicode(v)
+        validationErrors += six.text_type(v)
 
-    if validationErrors:
+    if validationErrors != u"":
         raise validate.ValidationException(validationErrors)
 
     return data, metadata
 
 
 def validate_doc(schema_names,  # type: Names
-                 doc,           # type: Union[Dict[unicode, Any], List[Dict[unicode, Any]], unicode]
+                 doc,           # type: Union[Dict[Text, Any], List[Dict[Text, Any]], Text, None]
                  loader,        # type: Loader
                  strict,        # type: bool
                  source=None
@@ -284,7 +297,7 @@ def validate_doc(schema_names,  # type: Names
 
     anyerrors = []
     for pos, item in enumerate(validate_doc):
-        sl = SourceLine(validate_doc, pos, unicode)
+        sl = SourceLine(validate_doc, pos, six.text_type)
         success = False
         for r in roots:
             success = validate.validate_ex(
@@ -294,7 +307,7 @@ def validate_doc(schema_names,  # type: Names
                 break
 
         if not success:
-            errors = []  # type: List[unicode]
+            errors = []  # type: List[Text]
             for r in roots:
                 if hasattr(r, "get_prop"):
                     name = r.get_prop(u"name")
@@ -323,16 +336,15 @@ def validate_doc(schema_names,  # type: Names
                     break
             anyerrors.append(u"%s\n%s" %
                              (objerr, validate.indent(bullets(errors, "- "))))
-    if anyerrors:
+    if len(anyerrors) > 0:
         raise validate.ValidationException(
             strip_dup_lineno(bullets(anyerrors, "* ")))
 
 
 def replace_type(items, spec, loader, found):
-    # type: (Any, Dict[unicode, Any], Loader, Set[unicode]) -> Any
+    # type: (Any, Dict[Text, Any], Loader, Set[Text]) -> Any
     """ Go through and replace types in the 'spec' mapping"""
 
-    items = copy.deepcopy(items)
     if isinstance(items, dict):
         # recursively check these fields for types to replace
         if "type" in items and items["type"] in ("record", "enum"):
@@ -342,6 +354,7 @@ def replace_type(items, spec, loader, found):
                 else:
                     found.add(items["name"])
 
+        items = copy.copy(items)
         for n in ("type", "items", "fields"):
             if n in items:
                 items[n] = replace_type(items[n], spec, loader, found)
@@ -352,7 +365,7 @@ def replace_type(items, spec, loader, found):
     elif isinstance(items, list):
         # recursively transform list
         return [replace_type(i, spec, loader, found) for i in items]
-    elif isinstance(items, (str, unicode)):
+    elif isinstance(items, (str, six.text_type)):
         # found a string which is a symbol corresponding to a type.
         replace_with = None
         if items in loader.vocab:
@@ -370,8 +383,8 @@ def replace_type(items, spec, loader, found):
 
 
 def avro_name(url):  # type: (AnyStr) -> AnyStr
-    doc_url, frg = urlparse.urldefrag(url)
-    if frg:
+    doc_url, frg = urllib.parse.urldefrag(url)
+    if frg != '':
         if '/' in frg:
             return frg[frg.rindex('/') + 1:]
         else:
@@ -379,19 +392,20 @@ def avro_name(url):  # type: (AnyStr) -> AnyStr
     return url
 
 
-Avro = TypeVar('Avro', Dict[unicode, Any], List[Any], unicode)
+Avro = TypeVar('Avro', Dict[Text, Any], List[Any], Text)
 
 
 def make_valid_avro(items,          # type: Avro
-                    alltypes,       # type: Dict[unicode, Dict[unicode, Any]]
-                    found,          # type: Set[unicode]
+                    alltypes,       # type: Dict[Text, Dict[Text, Any]]
+                    found,          # type: Set[Text]
                     union=False     # type: bool
                     ):
-    # type: (...) -> Union[Avro, Dict]
-    items = copy.deepcopy(items)
+    # type: (...) -> Union[Avro, Dict, Text]
     if isinstance(items, dict):
+        items = copy.copy(items)
         if items.get("name"):
-            items["name"] = avro_name(items["name"])
+            if items.get("inVocab", True):
+                items["name"] = avro_name(items["name"])
 
         if "type" in items and items["type"] in ("https://w3id.org/cwl/salad#record", "https://w3id.org/cwl/salad#enum", "record", "enum"):
             if (hasattr(items, "get") and items.get("abstract")) or ("abstract"
@@ -402,7 +416,7 @@ def make_valid_avro(items,          # type: Avro
                     "Named schemas must have a non-empty name: %s" % items)
 
             if items["name"] in found:
-                return items["name"]
+                return cast(Text, items["name"])
             else:
                 found.add(items["name"])
         for n in ("type", "items", "values", "fields"):
@@ -415,45 +429,57 @@ def make_valid_avro(items,          # type: Avro
     if isinstance(items, list):
         ret = []
         for i in items:
-            ret.append(make_valid_avro(i, alltypes, found, union=union))
+            ret.append(make_valid_avro(i, alltypes, found, union=union))  # type: ignore
         return ret
-    if union and isinstance(items, (str, unicode)):
+    if union and isinstance(items, six.string_types):
         if items in alltypes and avro_name(items) not in found:
             return cast(Dict, make_valid_avro(alltypes[items], alltypes, found,
                                               union=union))
         items = avro_name(items)
     return items
 
+def deepcopy_strip(item):  # type: (Any) -> Any
+    """Make a deep copy of list and dict objects.
+
+    Intentionally do not copy attributes.  This is to discard CommentedMap and
+    CommentedSeq metadata which is very expensive with regular copy.deepcopy.
+
+    """
+
+    if isinstance(item, dict):
+        return {k: deepcopy_strip(v) for k,v in six.iteritems(item)}
+    elif isinstance(item, list):
+        return [deepcopy_strip(k) for k in item]
+    else:
+        return item
 
 def extend_and_specialize(items, loader):
-    # type: (List[Dict[unicode, Any]], Loader) -> List[Dict[unicode, Any]]
+    # type: (List[Dict[Text, Any]], Loader) -> List[Dict[Text, Any]]
     """Apply 'extend' and 'specialize' to fully materialize derived record
     types."""
 
-    types = {}  # type: Dict[unicode, Any]
-    for t in items:
-        types[t["name"]] = t
+    items = deepcopy_strip(items)
+    types = {t["name"]: t for t in items}  # type: Dict[Text, Any]
     n = []
 
     for t in items:
-        t = copy.deepcopy(t)
         if "extends" in t:
-            spec = {}  # type: Dict[unicode, unicode]
+            spec = {}  # type: Dict[Text, Text]
             if "specialize" in t:
                 for sp in aslist(t["specialize"]):
                     spec[sp["specializeFrom"]] = sp["specializeTo"]
 
-            exfields = []  # type: List[unicode]
-            exsym = []  # type: List[unicode]
+            exfields = []  # type: List[Text]
+            exsym = []  # type: List[Text]
             for ex in aslist(t["extends"]):
                 if ex not in types:
                     raise Exception("Extends %s in %s refers to invalid base type" % (
                         t["extends"], t["name"]))
 
-                basetype = copy.deepcopy(types[ex])
+                basetype = copy.copy(types[ex])
 
                 if t["type"] == "record":
-                    if spec:
+                    if len(spec) > 0:
                         basetype["fields"] = replace_type(
                             basetype.get("fields", []), spec, loader, set())
 
@@ -466,10 +492,11 @@ def extend_and_specialize(items, loader):
                     exsym.extend(basetype.get("symbols", []))
 
             if t["type"] == "record":
+                t = copy.copy(t)
                 exfields.extend(t.get("fields", []))
                 t["fields"] = exfields
 
-                fieldnames = set()  # type: Set[unicode]
+                fieldnames = set()  # type: Set[Text]
                 for field in t["fields"]:
                     if field["name"] in fieldnames:
                         raise validate.ValidationException(
@@ -477,6 +504,7 @@ def extend_and_specialize(items, loader):
                     else:
                         fieldnames.add(field["name"])
             elif t["type"] == "enum":
+                t = copy.copy(t)
                 exsym.extend(t.get("symbols", []))
                 t["symbol"] = exsym
 
@@ -488,7 +516,7 @@ def extend_and_specialize(items, loader):
     for t in n:
         ex_types[t["name"]] = t
 
-    extended_by = {}  # type: Dict[unicode, unicode]
+    extended_by = {}  # type: Dict[Text, Text]
     for t in n:
         if "extends" in t:
             for ex in aslist(t["extends"]):
@@ -507,16 +535,15 @@ def extend_and_specialize(items, loader):
 
     return n
 
-
-def make_avro_schema(i,         # type: List[Dict[unicode, Any]]
+def make_avro_schema(i,         # type: List[Dict[Text, Any]]
                      loader     # type: Loader
                      ):
-    # type: (...) -> Tuple[Union[Names, SchemaParseException], List[Dict[unicode, Any]]]
+    # type: (...) -> Tuple[Union[Names, SchemaParseException], List[Dict[Text, Any]]]
     names = avro.schema.Names()
 
     j = extend_and_specialize(i, loader)
 
-    name_dict = {}  # type: Dict[unicode, Dict[unicode, Any]]
+    name_dict = {}  # type: Dict[Text, Dict[Text, Any]]
     for t in j:
         name_dict[t["name"]] = t
     j2 = make_valid_avro(j, name_dict, set())
