@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 import ruamel.yaml
 from ruamel.yaml.comments import CommentedBase, CommentedMap, CommentedSeq
 import re
@@ -5,6 +6,7 @@ import os
 
 from typing import (Any, AnyStr, Callable, cast, Dict, List, Iterable, Tuple,
                     TypeVar, Union, Text)
+import six
 
 lineno_re = re.compile(u"^(.*?:[0-9]+:[0-9]+: )(( *)(.*))")
 
@@ -15,19 +17,19 @@ def _add_lc_filename(r, source):  # type: (ruamel.yaml.comments.CommentedBase, A
         for d in r:
             _add_lc_filename(d, source)
     elif isinstance(r, dict):
-        for d in r.itervalues():
+        for d in six.itervalues(r):
             _add_lc_filename(d, source)
 
-def relname(source):  # type: (AnyStr) -> AnyStr
+def relname(source):  # type: (Text) -> Text
     if source.startswith("file://"):
         source = source[7:]
         source = os.path.relpath(source)
     return source
 
-def add_lc_filename(r, source):  # type: (ruamel.yaml.comments.CommentedBase, AnyStr) -> None
+def add_lc_filename(r, source):  # type: (ruamel.yaml.comments.CommentedBase, Text) -> None
     _add_lc_filename(r, relname(source))
 
-def reflow(text, maxline, shift=""):  # type: (AnyStr, int, AnyStr) -> AnyStr
+def reflow(text, maxline, shift=""):  # type: (Text, int, Text) -> Text
     if maxline < 20:
         maxline = 20
     if len(text) > maxline:
@@ -46,7 +48,7 @@ def indent(v, nolead=False, shift=u"  ", bullet=u"  "):  # type: (Text, bool, Te
     else:
         def lineno(i, l):  # type: (int, Text) -> Text
             r = lineno_re.match(l)
-            if r:
+            if bool(r):
                 return r.group(1) + (bullet if i == 0 else shift) + r.group(2)
             else:
                 return (bullet if i == 0 else shift) + l
@@ -79,7 +81,7 @@ def strip_dup_lineno(text, maxline=None):  # type: (Text, int) -> Text
             msg.append(" " * len(g.group(1)) + g2)
     return "\n".join(msg)
 
-def cmap(d, lc=None, fn=None):  # type: (Union[int, float, str, unicode, Dict, List], List[int], unicode) -> Union[int, float, str, unicode, CommentedMap, CommentedSeq]
+def cmap(d, lc=None, fn=None):  # type: (Union[int, float, str, Text, Dict, List], List[int], Text) -> Union[int, float, str, Text, CommentedMap, CommentedSeq]
     if lc is None:
         lc = [0, 0, 0, 0]
     if fn is None:
@@ -87,7 +89,7 @@ def cmap(d, lc=None, fn=None):  # type: (Union[int, float, str, unicode, Dict, L
 
     if isinstance(d, CommentedMap):
         fn = d.lc.filename if hasattr(d.lc, "filename") else fn
-        for k,v in d.iteritems():
+        for k,v in six.iteritems(d):
             if k in d.lc.data:
                 d[k] = cmap(v, lc=d.lc.data[k], fn=fn)
             else:
@@ -132,33 +134,40 @@ def cmap(d, lc=None, fn=None):  # type: (Union[int, float, str, unicode, Dict, L
         return d
 
 class SourceLine(object):
-    def __init__(self, item, key=None, raise_type=unicode):  # type: (Any, Any, Callable) -> None
+    def __init__(self, item, key=None, raise_type=six.text_type):  # type: (Any, Any, Callable) -> None
         self.item = item
         self.key = key
         self.raise_type = raise_type
 
-    def __enter__(self):
+    def __enter__(self):  # type: () -> SourceLine
         return self
 
-    def __exit__(self, exc_type, exc_value, traceback):
+    def __exit__(self,
+                 exc_type,   # type: Any
+                 exc_value,  # type: Any
+                 traceback   # type: Any
+                 ):  # -> Any
         if not exc_value:
             return
-        raise self.makeError(unicode(exc_value))
+        raise self.makeError(six.text_type(exc_value))
+
+    def makeLead(self):  # type: () -> Text
+        if self.key is None or self.item.lc.data is None or self.key not in self.item.lc.data:
+            return "%s:%i:%i:" % (self.item.lc.filename if hasattr(self.item.lc, "filename") else "",
+                                  (self.item.lc.line or 0)+1,
+                                  (self.item.lc.col or 0)+1)
+        else:
+            return "%s:%i:%i:" % (self.item.lc.filename if hasattr(self.item.lc, "filename") else "",
+                                  (self.item.lc.data[self.key][0] or 0)+1,
+                                  (self.item.lc.data[self.key][1] or 0)+1)
 
     def makeError(self, msg):  # type: (Text) -> Any
         if not isinstance(self.item, ruamel.yaml.comments.CommentedBase):
             return self.raise_type(msg)
         errs = []
-        if self.key is None or self.item.lc.data is None or self.key not in self.item.lc.data:
-            lead = "%s:%i:%i:" % (self.item.lc.filename,
-                                  self.item.lc.line+1,
-                                  self.item.lc.col+1)
-        else:
-            lead = "%s:%i:%i:" % (self.item.lc.filename,
-                                  self.item.lc.data[self.key][0]+1,
-                                  self.item.lc.data[self.key][1]+1)
+        lead = self.makeLead()
         for m in msg.splitlines():
-            if lineno_re.match(m):
+            if bool(lineno_re.match(m)):
                 errs.append(m)
             else:
                 errs.append("%s %s" % (lead, m))
