@@ -1,15 +1,16 @@
+from __future__ import absolute_import
+from __future__ import print_function
 from .util import get_data
 import unittest
-from typing import cast
 from schema_salad.schema import load_schema, load_and_validate
 from schema_salad.validate import ValidationException
 from avro.schema import Names
+import six
 
 class TestErrors(unittest.TestCase):
     def test_errors(self):
         document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
             get_data(u"tests/test_schema/CommonWorkflowLanguage.yml"))
-        avsc_names = cast(Names, avsc_names)
 
         for t in ("test_schema/test1.cwl",
                   "test_schema/test2.cwl",
@@ -21,11 +22,14 @@ class TestErrors(unittest.TestCase):
                   "test_schema/test8.cwl",
                   "test_schema/test9.cwl",
                   "test_schema/test10.cwl",
-                  "test_schema/test11.cwl"):
+                  "test_schema/test11.cwl",
+                  "test_schema/test12.cwl",
+                  "test_schema/test13.cwl",
+                  "test_schema/test14.cwl"):
             with self.assertRaises(ValidationException):
                 try:
                     load_and_validate(document_loader, avsc_names,
-                            unicode(get_data("tests/"+t)), True)
+                            six.text_type(get_data("tests/"+t)), True)
                 except ValidationException as e:
-                    print "\n", e
+                    print("\n", e)
                     raise
