@@ -1,12 +1,19 @@
+from __future__ import absolute_import
 import pprint
 import avro.schema
 from avro.schema import Schema
 import sys
-import urlparse
 import re
-from typing import Any, Union
+import logging
+
+import six
+from six.moves import urllib
+from six.moves import range
+
+from typing import Any, List, Set, Union, Text
 from .sourceline import SourceLine, lineno_re, bullets, indent
 
+_logger = logging.getLogger("salad")
 
 class ValidationException(Exception):
     pass
@@ -18,9 +25,9 @@ class ClassValidationException(ValidationException):
 
 def validate(expected_schema,           # type: Schema
              datum,                     # type: Any
-             identifiers=set(),         # type: Set[unicode]
+             identifiers=[],            # type: List[Text]
              strict=False,              # type: bool
-             foreign_properties=set()   # type: Set[unicode]
+             foreign_properties=set()   # type: Set[Text]
              ):
     # type: (...) -> bool
     return validate_ex(
@@ -54,18 +61,20 @@ def vpformat(datum):  # type: (Any) -> str
     return a
 
 
-def validate_ex(expected_schema,            # type: Schema
-                datum,                      # type: Any
-                identifiers=None,           # type: Set[unicode]
-                strict=False,               # type: bool
-                foreign_properties=None,    # type: Set[unicode]
-                raise_ex=True               # type: bool
+def validate_ex(expected_schema,                  # type: Schema
+                datum,                            # type: Any
+                identifiers=None,                 # type: List[Text]
+                strict=False,                     # type: bool
+                foreign_properties=None,          # type: Set[Text]
+                raise_ex=True,                    # type: bool
+                strict_foreign_properties=False,  # type: bool
+                logger=_logger                    # type: logging.Logger
                 ):
     # type: (...) -> bool
     """Determine if a python datum is an instance of a schema."""
 
     if not identifiers:
-        identifiers = set()
+        identifiers = []
 
     if not foreign_properties:
         foreign_properties = set()
@@ -89,7 +98,7 @@ def validate_ex(expected_schema,            # type: Schema
             else:
                 return False
     elif schema_type == 'string':
-        if isinstance(datum, basestring):
+        if isinstance(datum, six.string_types):
             return True
         elif isinstance(datum, bytes):
             datum = datum.decode(u"utf-8")
@@ -109,7 +118,7 @@ def validate_ex(expected_schema,            # type: Schema
             else:
                 return False
     elif schema_type == 'int':
-        if ((isinstance(datum, int) or isinstance(datum, long))
+        if (isinstance(datum, six.integer_types)
                 and INT_MIN_VALUE <= datum <= INT_MAX_VALUE):
             return True
         else:
@@ -118,7 +127,7 @@ def validate_ex(expected_schema,            # type: Schema
             else:
                 return False
     elif schema_type == 'long':
-        if ((isinstance(datum, int) or isinstance(datum, long))
+        if ((isinstance(datum, six.integer_types))
                 and LONG_MIN_VALUE <= datum <= LONG_MAX_VALUE):
             return True
         else:
@@ -128,7 +137,7 @@ def validate_ex(expected_schema,            # type: Schema
             else:
                 return False
     elif schema_type in ['float', 'double']:
-        if (isinstance(datum, int) or isinstance(datum, long)
+        if (isinstance(datum, six.integer_types)
                 or isinstance(datum, float)):
             return True
         else:
@@ -146,7 +155,7 @@ def validate_ex(expected_schema,            # type: Schema
                     raise ValidationException(u"'Any' type must be non-null")
                 else:
                     return False
-        if not isinstance(datum, basestring):
+        if not isinstance(datum, six.string_types):
             if raise_ex:
                 raise ValidationException(
                     u"value is a %s but expected a string" % (type(datum).__name__))
@@ -167,39 +176,44 @@ def validate_ex(expected_schema,            # type: Schema
             for i, d in enumerate(datum):
                 try:
                     sl = SourceLine(datum, i, ValidationException)
-                    if not validate_ex(expected_schema.items, d, identifiers, strict=strict,
+                    if not validate_ex(expected_schema.items, d, identifiers,
+                                       strict=strict,
                                        foreign_properties=foreign_properties,
-                                       raise_ex=raise_ex):
+                                       raise_ex=raise_ex,
+                                       strict_foreign_properties=strict_foreign_properties,
+                                       logger=logger):
                         return False
                 except ValidationException as v:
                     if raise_ex:
                         raise sl.makeError(
-                            unicode("item is invalid because\n%s" % (indent(str(v)))))
+                            six.text_type("item is invalid because\n%s" % (indent(str(v)))))
                     else:
                         return False
             return True
         else:
             if raise_ex:
-                raise ValidationException(u"the value is not a list, expected list of %s" % (
-                    friendly(expected_schema.items)))
+                raise ValidationException(u"the value %s is not a list, expected list of %s" % (
+                    vpformat(datum), friendly(expected_schema.items)))
             else:
                 return False
     elif isinstance(expected_schema, avro.schema.UnionSchema):
         for s in expected_schema.schemas:
-            if validate_ex(s, datum, identifiers, strict=strict, raise_ex=False):
+            if validate_ex(s, datum, identifiers, strict=strict, raise_ex=False,
+                           strict_foreign_properties=strict_foreign_properties,
+                           logger=logger):
                 return True
 
         if not raise_ex:
             return False
 
-        errors = []  # type: List[unicode]
+        errors = []  # type: List[Text]
         checked = []
         for s in expected_schema.schemas:
             if isinstance(datum, list) and not isinstance(s, avro.schema.ArraySchema):
                 continue
             elif isinstance(datum, dict) and not isinstance(s, avro.schema.RecordSchema):
                 continue
-            elif isinstance(datum, (bool, int, long, float, basestring)) and isinstance(s, (avro.schema.ArraySchema, avro.schema.RecordSchema)):
+            elif isinstance(datum, (bool, six.integer_types, float, six.string_types)) and isinstance(s, (avro.schema.ArraySchema, avro.schema.RecordSchema)):
                 continue
             elif datum is not None and s.type == "null":
                 continue
@@ -207,12 +221,15 @@ def validate_ex(expected_schema,            # type: Schema
             checked.append(s)
             try:
                 validate_ex(s, datum, identifiers, strict=strict,
-                            foreign_properties=foreign_properties, raise_ex=True)
+                            foreign_properties=foreign_properties,
+                            raise_ex=True,
+                            strict_foreign_properties=strict_foreign_properties,
+                            logger=logger)
             except ClassValidationException as e:
                 raise
             except ValidationException as e:
-                errors.append(unicode(e))
-        if errors:
+                errors.append(six.text_type(e))
+        if bool(errors):
             raise ValidationException(bullets(["tried %s but\n%s" % (friendly(
                 checked[i]), indent(errors[i])) for i in range(0, len(errors))], "- "))
         else:
@@ -237,7 +254,11 @@ def validate_ex(expected_schema,            # type: Schema
                     else:
                         return False
                 if expected_schema.name != d:
-                    return False
+                    if raise_ex:
+                        raise ValidationException(
+                            u"Expected class '%s' but this is '%s'" % (expected_schema.name, d))
+                    else:
+                        return False
                 classmatch = d
                 break
 
@@ -255,9 +276,12 @@ def validate_ex(expected_schema,            # type: Schema
                     fieldval = None
 
             try:
-                sl = SourceLine(datum, f.name, unicode)
-                if not validate_ex(f.type, fieldval, identifiers, strict=strict, foreign_properties=foreign_properties,
-                                   raise_ex=raise_ex):
+                sl = SourceLine(datum, f.name, six.text_type)
+                if not validate_ex(f.type, fieldval, identifiers, strict=strict,
+                                   foreign_properties=foreign_properties,
+                                   raise_ex=raise_ex,
+                                   strict_foreign_properties=strict_foreign_properties,
+                                   logger=logger):
                     return False
             except ValidationException as v:
                 if f.name not in datum:
@@ -266,26 +290,36 @@ def validate_ex(expected_schema,            # type: Schema
                     errors.append(sl.makeError(u"the `%s` field is not valid because\n%s" % (
                         f.name, indent(str(v)))))
 
-        if strict:
-            for d in datum:
-                found = False
-                for f in expected_schema.fields:
-                    if d == f.name:
-                        found = True
-                if not found:
-                    sl = SourceLine(datum, d, unicode)
-                    if d not in identifiers and d not in foreign_properties and d[0] not in ("@", "$"):
-                        if not raise_ex:
-                            return False
-                        split = urlparse.urlsplit(d)
-                        if split.scheme:
-                            errors.append(sl.makeError(
-                                u"unrecognized extension field `%s` and strict is True.  Did you include a $schemas section?" % (d)))
+        for d in datum:
+            found = False
+            for f in expected_schema.fields:
+                if d == f.name:
+                    found = True
+            if not found:
+                sl = SourceLine(datum, d, six.text_type)
+                if d not in identifiers and d not in foreign_properties and d[0] not in ("@", "$"):
+                    if (d not in identifiers and strict) and (
+                            d not in foreign_properties and strict_foreign_properties) and not raise_ex:
+                        return False
+                    split = urllib.parse.urlsplit(d)
+                    if split.scheme:
+                        err = sl.makeError(u"unrecognized extension field `%s`%s."
+                                           "  Did you include "
+                                           "a $schemas section?" % (
+                                               d, " and strict_foreign_properties is True" if strict_foreign_properties else ""))
+                        if strict_foreign_properties:
+                            errors.append(err)
                         else:
-                            errors.append(sl.makeError(u"invalid field `%s`, expected one of: %s" % (
-                                d, ", ".join("'%s'" % fn.name for fn in expected_schema.fields))))
+                            logger.warn(err)
+                    else:
+                        err = sl.makeError(u"invalid field `%s`, expected one of: %s" % (
+                            d, ", ".join("'%s'" % fn.name for fn in expected_schema.fields)))
+                        if strict:
+                            errors.append(err)
+                        else:
+                            logger.warn(err)
 
-        if errors:
+        if bool(errors):
             if raise_ex:
                 if classmatch:
                     raise ClassValidationException(bullets(errors, "* "))
